﻿using Stemmer.Cvb;
using Stemmer.Cvb.Async;
using Stemmer.Cvb.Driver;
using Stemmer.Cvb.Wpf;
using System;
using System.Windows;

namespace WpfAviPlayback
{
  /// <summary>
  /// Interaction logic for MainWindow.xaml
  /// </summary>
  /// <remarks>
  /// Here is only controller code. All view related connections are done
  /// in <see cref="MainWindow.xaml"/>.
  /// </remarks>
  public partial class MainWindow : Window, IDisposable
  {
    #region Creation and Clean-up

    /// <summary>
    /// Creates the main window.
    /// </summary>
    public MainWindow()
    {
      InitializeComponent();
    }

    /// <summary>
    /// Finalizer.
    /// </summary>
    ~MainWindow()
    {
      Dispose(false);
    }

    /// <summary>
    /// Disposes this object.
    /// </summary>
    public void Dispose()
    {
      Dispose(true);
      GC.SuppressFinalize(this);
    }

    /// <summary>
    /// Dispose handler.
    /// </summary>
    /// <param name="disposing"><b>true</b> if called via 
    /// <see cref="IDisposable.Dispose"/>; <b>false</b> if called by finalizer.
    /// </param>
    protected virtual void Dispose(bool disposing)
    {
      if (disposing)
      {
        Device?.Dispose();
      }
    }

    // Stops acquisition if running.
    private void mainWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
    {
      menuGrab.IsChecked = false;
    }

    #endregion

    #region Dependency Properties

    #region Device 

    /// <summary>
    /// Gets or sets the current CVB device for acquiring images and feature 
    /// control.
    /// </summary>
    /// <remarks>
    /// We need a dependency property as we want to bind our display to this.
    /// </remarks>
    public Device Device
    {
      get { return (Device)GetValue(DeviceProperty); }
      set { SetValue(DeviceProperty, value); }
    }

    // Using a DependencyProperty as the backing store for Device.  This enables animation, styling, binding, etc...
    public static readonly DependencyProperty DeviceProperty =
        DependencyProperty.Register("Device", typeof(Device), typeof(MainWindow), new PropertyMetadata(null, Device_Changed));

    // Cleans-up old devices and notifies HasStreams and NodeMaps
    private static void Device_Changed(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      if (!ReferenceEquals(e.OldValue, e.NewValue))
      {
        var oldDevice = e.OldValue as Device;
        oldDevice?.Dispose();

        d.CoerceValue(HasStreamProperty);
      }
    }

    #endregion

    #region HasStream

    /// <summary>
    /// Gets whether this <see cref="Device"/> has a stream.
    /// </summary>
    public bool HasStream
    {
      get { return (bool)GetValue(HasStreamProperty); }
    }

    // The read-only DependencyProperty key for HasStream.
    private static readonly DependencyPropertyKey HasStreamPropertyKey =
        DependencyProperty.RegisterReadOnly("HasStream", typeof(bool), typeof(MainWindow), new PropertyMetadata(false, null, HasStream_Coerce));
    // Using a DependencyProperty as the backing store for HasStream.  This enables animation, styling, binding, etc...
    public static readonly DependencyProperty HasStreamProperty = HasStreamPropertyKey.DependencyProperty;

    private static object HasStream_Coerce(DependencyObject d, object baseValue)
    {
      var device = (Device)d.GetValue(DeviceProperty);

      return device?.Stream != null;
    }



    #endregion

    #endregion

    #region Open Handling

    // Called when the Open menu item is clicked.
    private void menuOpen_Click(object sender, RoutedEventArgs e)
    {
      try
      {
        var device = FileDialogs.LoadByDialog(path => DeviceFactory.Open(path), "Audio Video Interleaved Files (*.avi)|*.avi");
        if (device == null)
          return; // canceled

        Device = device; // this also updates HasStream and NodeMaps 
      }
      catch (Exception ex)
      {
        MessageBox.Show(this, ex.Message, "Error loading driver", MessageBoxButton.OK, MessageBoxImage.Error);
      }
    }

    #endregion

    #region Acquisition Handling

    // Called when the Grab menu changes to checked.
    private async void menuGrab_Checked(object sender, RoutedEventArgs e)
    {
      Device.Stream.Start();
      menuOpen.IsEnabled = false;
      try
      {
        while (menuGrab.IsChecked)
        {
          using (StreamImage image = await Device.Stream.WaitAsync())
          {
            // display is notified via Device.DeviceImage.PixelContentChanged

            // add processing here...
          }
        }
      }
      catch (OperationCanceledException)
      {
        // acquisition was aborted
      }
      finally
      {
        menuOpen.IsEnabled = true;
      }
    }

    // Called when the Grab menu changes to unchecked.
    private void menuGrab_Unchecked(object sender, RoutedEventArgs e)
    {
      if (Device?.Stream?.IsRunning ?? false)
        Device.Stream.Abort();
    }

    // Called when the Snap menu was clicked.
    private void menuSnap_Click(object sender, RoutedEventArgs e)
    {
      menuGrab.IsChecked = false;

      Device.Stream.GetSnapshot();
    }

    #endregion
  }
}
